package com.payneteasy.telpo.nfc.aidl;

import com.payneteasy.telpo.nfc.aidl.ByteArrayResponse;
import com.payneteasy.telpo.nfc.aidl.NfcResponse;

/**
 * Access to Telpo NFC service
 */
interface IRemoteNfcService {

    /**
     * Service version
     *
     * @return String with version
     */
    String getVersion();

    /**
     * Session id to find in logs
     *
     * @return session id
     */
    String getSessionId();

    /**
     * Opens NFC device
     *
     * @return {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND}
     */
    NfcResponse open();

    /**
     * Closes NFC device
     *
     * @return {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND}
     */
    NfcResponse close();

    /**
     * Selects an NFC card and read basic information
     *
     * @param aTimeoutMs timeout time in millisecond
     * 
     * @return byte array with response codes {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_OPENED}
     */
    ByteArrayResponse waitForCard(int aTimeoutMs);

    /**
     * Performs first authenticate
     *
     * @param aBlockNumber Key Block number
     * @param aKeyType     could be 0x0A or 0x0B
     * @param aAesKey      16-bytes key data
     *
     * @return {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_OPENED}
     */
    NfcResponse mifarePlusSl3Authenicate(byte aBlockNumber, byte aKeyType, in byte[] aAesKey);

    /**
     * Reads data from the given block
     *
     * @param aBlockNumber block number
     *
     * @return 16 bytes array with response codes {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_OPENED}
     */
    ByteArrayResponse mifarePlusSl3Read(byte aBlockNumber);

    /**
     * Writes the data to the given block number
     *
     * @param aBlockNumber number of the block to be written
     * @param aData        16-bytes data
     * 
     * @return {@link NfcErrorCodes#NFC_RESULT_OK},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_FOUND},
     *         {@link NfcErrorCodes#NFC_ERROR_DEVICE_NOT_OPENED}
     */
    NfcResponse mifarePlusSl3Write(byte aBlockNumber, in byte[] aData);

}
